//
//  CPDFViewController.m
//   Edit-Ctrl-Demo
//
//  Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
//
//  THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
//  AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
//  UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
//  This notice may not be removed from this file.
//

#import "CPDFViewController.h"

#import <ComPDFKit/ComPDFKit.h>
#import <compdfkit_tools/compdfkit_tools.h>

@interface CPDFViewController () <CPDFSoundPlayBarDelegate,CPDFAnnotationBarDelegate,CPDFToolsViewControllerDelegate,CPDFNoteOpenViewControllerDelegate,CPDFBOTAViewControllerDelegate,CPDFEditToolBarDelegate>

@property(nonatomic, strong) CPDFAnnotationBar *annotationBar;

@property(nonatomic, strong) CPDFSoundPlayBar *soundPlayBar;

@property(nonatomic, strong) CAnnotationManage *annotationManage;

@property(nonatomic, strong) CPDFEditToolBar * toolBar;

@property(nonatomic, strong) CPDFEditViewController *baseVC;

@property(nonatomic, assign) CPDFEditMode editMode;

@end

@implementation CPDFViewController


- (void)viewDidLoad {
    [super viewDidLoad];
    
   self.pdfListView.toolModel = CToolModelViewer;
    
    CPDFEditingConfig *editingConfig = [[CPDFEditingConfig alloc]init];
    editingConfig.editingBorderWidth = 1.0;
    editingConfig.editingOffsetGap = 5;
    self.pdfListView.editingConfig = editingConfig;
    
    [self initAnnotationBar];
    
    [self initWithEditTool];

    [self enterViewerMode];
}

- (void)initAnnotationBar {
    self.annotationManage = [[CAnnotationManage alloc] initWithPDFView:self.pdfListView];
        
    self.annotationBar = [[CPDFAnnotationBar alloc] initAnnotationManage:self.annotationManage];
    
    CGFloat height = 44.0;
    if (@available(iOS 11.0, *))
        height += self.view.safeAreaInsets.bottom;
    
    self.annotationBar.frame = CGRectMake(0, self.view.frame.size.height - height, self.view.frame.size.width, height);
    self.annotationBar.delegate = self;
    [self.annotationBar setParentVC:self];
    [self.view addSubview:self.annotationBar];
}

- (void)initWithEditTool {
    if(!self.toolBar){
        self.toolBar = [[CPDFEditToolBar alloc] initWithPDFView:self.pdfListView];
    }
    
    self.toolBar.delegate = self;
    [self.view addSubview:self.toolBar];
}

- (void)initWitNavigationTitle {
    //titleButton
    CNavigationBarTitleButton * navTitleButton = [[CNavigationBarTitleButton alloc] init];
    self.titleButton = navTitleButton;
    self.navigationTitle = NSLocalizedString(@"View", nil);
    [navTitleButton setImage:[UIImage imageNamed:@"syasarrow"] forState:UIControlStateNormal];
    [navTitleButton addTarget:self action:@selector(titleButtonClickd:) forControlEvents:UIControlEventTouchUpInside];
    [navTitleButton setTitle:self.navigationTitle forState:UIControlStateNormal];
    [navTitleButton setTitleColor:[CPDFColorUtils CAnyReverseBackgooundColor] forState:UIControlStateNormal];
    self.titleButton.frame = CGRectMake(0, 0, 50, 30);
    self.navigationItem.titleView = self.titleButton;
}

- (void)viewWillLayoutSubviews {
    [super viewWillLayoutSubviews];
    if([self.popMenu superview]) {
        if (@available(iOS 11.0, *)) {
            [self.popMenu showMenuInRect:CGRectMake(self.view.frame.size.width - self.view.safeAreaInsets.right - 250, CGRectGetMaxY(self.navigationController.navigationBar.frame), 250, 200)];
        } else {
            // Fallback on earlier versions
            [self.popMenu showMenuInRect:CGRectMake(self.view.frame.size.width - 250, CGRectGetMaxY(self.navigationController.navigationBar.frame), 250, 200)];
        }
    }
    
    CGFloat height = 44.0;
    
    if (@available(iOS 11.0, *))
        height += self.view.safeAreaInsets.bottom;

    if(CToolModelAnnotation == self.pdfListView.toolModel) {
        self.annotationBar.frame = CGRectMake(0, self.view.frame.size.height - height, self.view.frame.size.width, height);
    } else if(CToolModelEdit == self.pdfListView.toolModel){
        self.toolBar.frame = CGRectMake(0, self.view.frame.size.height - height, self.view.frame.size.width, height);
    }else{
        self.annotationBar.frame = CGRectMake(0, self.view.bounds.size.height, self.view.frame.size.width, height);
    }
    
    CGFloat tPosY = 0;
    CGFloat tBottomY = 0;

    if(CToolModelAnnotation == self.pdfListView.toolModel) {
        if (!self.navigationController.navigationBarHidden) {
            [UIView animateWithDuration:0.3 animations:^{
                CGRect frame = self.annotationBar.frame;
                frame.origin.y = self.view.bounds.size.height-frame.size.height;
                self.annotationBar.frame = frame;
            }];
            CGRect rectStatus = [[UIApplication sharedApplication] statusBarFrame];
            tPosY = self.navigationController.navigationBar.frame.size.height + rectStatus.size.height;
            
            tBottomY = self.annotationBar.frame.size.height;
        } else {
            [UIView animateWithDuration:0.3 animations:^{
                CGRect frame = self.annotationBar.frame;
                frame.origin.y = self.view.bounds.size.height;
                self.annotationBar.frame = frame;
            }];
        }
    } else {
        tPosY = 0;
        if (!self.navigationController.navigationBarHidden) {
            CGRect rectStatus = [[UIApplication sharedApplication] statusBarFrame];
            tPosY = self.navigationController.navigationBar.frame.size.height + rectStatus.size.height;
        }
    }
    
    if (@available(iOS 11.0, *)) {
        self.pdfListView.frame = CGRectMake(self.view.safeAreaInsets.left, tPosY, self.view.frame.size.width - self.view.safeAreaInsets.left - self.view.safeAreaInsets.right, self.view.frame.size.height - tBottomY - tPosY);
    } else {
        self.pdfListView.frame = CGRectMake(0, tPosY, self.view.frame.size.width, self.view.frame.size.height  - tBottomY - tPosY);
    }
}

#pragma mark - Private

- (void)enterEditMode {
    self.toolBar.hidden = NO;
    self.annotationBar.hidden = YES;
    self.pdfListView.toolModel = CToolModelEdit;
    [self.pdfListView beginEditingLoadType:CEditingLoadTypeText | CEditingLoadTypeImage];
    self.navigationTitle = NSLocalizedString(@"Edit", nil);
    [self.titleButton setTitle:self.navigationTitle forState:UIControlStateNormal];
    
    [self.toolBar updateButtonState];
    
    CGFloat tPosY = 0;
    CGFloat tBottomY = 0;
    CGRect frame = self.toolBar.frame;
    frame.origin.y = self.view.bounds.size.height-frame.size.height;
    self.toolBar.frame = frame;
    
    CGRect rectStatus = [[UIApplication sharedApplication] statusBarFrame];
    tPosY = self.navigationController.navigationBar.frame.size.height + rectStatus.size.height;
    tBottomY = self.toolBar.frame.size.height;
    
    if (@available(iOS 11.0, *)) {
        self.pdfListView.frame = CGRectMake(self.view.safeAreaInsets.left, tPosY, self.view.frame.size.width - self.view.safeAreaInsets.left - self.view.safeAreaInsets.right, self.view.frame.size.height - tBottomY - tPosY);
    } else {
        self.pdfListView.frame = CGRectMake(0, tPosY, self.view.frame.size.width, self.view.frame.size.height  - tBottomY - tPosY);
    }
}

- (void)enterAnnotationMode {
    self.toolBar.hidden = YES;
    self.annotationBar.hidden = NO;
    if (self.pdfListView.isEdited) {
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
            [self.pdfListView commitEditing];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.pdfListView endOfEditing];
            });
        });
    } else {
        [self.pdfListView endOfEditing];
    }
    self.pdfListView.toolModel = CToolModelAnnotation;
    self.navigationTitle = NSLocalizedString(@"Annotation", nil);
    [self.titleButton setTitle:self.navigationTitle forState:UIControlStateNormal];
    
    CGFloat tPosY = 0;
    CGFloat tBottomY = 0;
    CGRect frame = self.annotationBar.frame;
    frame.origin.y = self.view.bounds.size.height-frame.size.height;
    self.annotationBar.frame = frame;
    
    CGRect rectStatus = [[UIApplication sharedApplication] statusBarFrame];
    tPosY = self.navigationController.navigationBar.frame.size.height + rectStatus.size.height;
    tBottomY = self.annotationBar.frame.size.height;
    
    if (@available(iOS 11.0, *)) {
        self.pdfListView.frame = CGRectMake(self.view.safeAreaInsets.left, tPosY, self.view.frame.size.width - self.view.safeAreaInsets.left - self.view.safeAreaInsets.right, self.view.frame.size.height - tBottomY - tPosY);
    } else {
        self.pdfListView.frame = CGRectMake(0, tPosY, self.view.frame.size.width, self.view.frame.size.height  - tBottomY - tPosY);
    }
}

- (void)enterViewerMode {
    self.toolBar.hidden = YES;
    if (self.pdfListView.isEdited) {
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
            [self.pdfListView commitEditing];
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.pdfListView endOfEditing];
            });
        });
    } else {
        [self.pdfListView endOfEditing];
    }
    self.pdfListView.toolModel = CToolModelViewer;
    self.navigationTitle = NSLocalizedString(@"Viewer", nil);
    [self.titleButton setTitle:self.navigationTitle forState:UIControlStateNormal];
    
    CGRect frame = self.annotationBar.frame;
    frame.origin.y = self.view.bounds.size.height;
    self.annotationBar.frame = frame;
    
    CGFloat tPosY = 0;
    CGFloat tBottomY = 0;
    
    if (@available(iOS 11.0, *)) {
        self.pdfListView.frame = CGRectMake(self.view.safeAreaInsets.left, tPosY, self.view.frame.size.width - self.view.safeAreaInsets.left - self.view.safeAreaInsets.right, self.view.frame.size.height - tBottomY - tPosY);
    } else {
        self.pdfListView.frame = CGRectMake(0, tPosY, self.view.frame.size.width, self.view.frame.size.height  - tBottomY - tPosY);
    }
}

- (void)enterPDFShare  {
    if(self.pdfListView.isEditing && self.pdfListView.isEdited) {
        [self.pdfListView commitEditing];
        [self.pdfListView endOfEditing];
    }
    
    [super enterPDFShare];
}

- (void)enterPDFAddFile  {
    if(self.pdfListView.isEditing && self.pdfListView.isEdited) {
        [self.pdfListView commitEditing];
        [self.pdfListView endOfEditing];
    }
    [super enterPDFAddFile];
}

#pragma mark - Action

- (void)navigationRightItemBota{
    CPDFBOTAViewController *botaViewController = nil;

    if(CToolModelAnnotation == self.pdfListView.toolModel) {
        botaViewController = [[CPDFBOTAViewController alloc] initCustomizeWithPDFView:self.pdfListView navArrays:@[@(CPDFBOTATypeStateOutline),@(CPDFBOTATypeStateBookmark),@(CPDFBOTATypeStateAnnotation)]];

    } else {
        botaViewController = [[CPDFBOTAViewController alloc] initCustomizeWithPDFView:self.pdfListView navArrays:@[@(CPDFBOTATypeStateOutline),@(CPDFBOTATypeStateBookmark)]];
    }
    botaViewController.delegate = self;
    
    AAPLCustomPresentationController *presentationController NS_VALID_UNTIL_END_OF_SCOPE;
   
    presentationController = [[AAPLCustomPresentationController alloc] initWithPresentedViewController:botaViewController presentingViewController:self];
    botaViewController.transitioningDelegate = presentationController;
    
    [self presentViewController:botaViewController animated:YES completion:nil];
}

- (void) titleButtonClickd:(UIButton *) button {
    CPDFToolsViewController * toolsVc = [[CPDFToolsViewController alloc] initCustomizeWithToolArrays:@[@(CToolModelViewer),@(CToolModelEdit),@(CToolModelAnnotation)]];
    toolsVc.delegate = self;
    AAPLCustomPresentationController *presentationController NS_VALID_UNTIL_END_OF_SCOPE;
    presentationController = [[AAPLCustomPresentationController alloc] initWithPresentedViewController:toolsVc presentingViewController:self];
    toolsVc.transitioningDelegate = presentationController;
    [self presentViewController:toolsVc animated:YES completion:nil];
}

#pragma - CPDFEditToolBarDelegate

- (void)undoDidClickInToolBar:(CPDFEditToolBar *)toolBar{
    [self.pdfListView editTextUndo];
}

- (void)redoDidClickInToolBar:(CPDFEditToolBar *)toolBar{
    [self.pdfListView editTextRedo];
}

- (void)propertyEditDidClickInToolBar:(CPDFEditToolBar *)toolBar{
    [self showMenuList];
}


- (void)showMenuList {
    _baseVC = [[CPDFEditViewController alloc] initWithPDFView:self.pdfListView];
    _baseVC.editMode = self.editMode;
    if((self.editMode == CPDFEditModeText || self.editMode == CPDFEditModeImage) && self.pdfListView.editStatus != CEditingSelectStateEmpty){
        
        AAPLCustomPresentationController *presentationController NS_VALID_UNTIL_END_OF_SCOPE;
       
        presentationController = [[AAPLCustomPresentationController alloc] initWithPresentedViewController:self.baseVC presentingViewController:self];
        self.baseVC.transitioningDelegate = presentationController;
        
        [self presentViewController:self.baseVC animated:YES completion:nil];
    }
}

#pragma mark - CPDFListViewDelegate

- (void)PDFListViewPerformTouchEnded:(CPDFListView *)pdfView {
    CGFloat tPosY = 0;
    CGFloat tBottomY = 0;

    if(CToolModelAnnotation == self.pdfListView.toolModel) {
        if (self.navigationController.navigationBarHidden) {
            [self.navigationController setNavigationBarHidden:NO animated:YES];
            [UIView animateWithDuration:0.3 animations:^{
                CGRect frame = self.annotationBar.frame;
                frame.origin.y = self.view.bounds.size.height-frame.size.height;
                self.annotationBar.frame = frame;
                self.pdfListView.pageSliderView.alpha = 1.0;
                self.annotationBar.topToolBar.alpha = 1.0;
                self.annotationBar.drawPencilFuncView.alpha = 1.0;
            }];
            CGRect rectStatus = [[UIApplication sharedApplication] statusBarFrame];
            tPosY = self.navigationController.navigationBar.frame.size.height + rectStatus.size.height;
            
            tBottomY = self.annotationBar.frame.size.height;
        } else {
            [self.navigationController setNavigationBarHidden:YES animated:YES];
            [UIView animateWithDuration:0.3 animations:^{
                CGRect frame = self.annotationBar.frame;
                frame.origin.y = self.view.bounds.size.height;
                self.annotationBar.frame = frame;
                self.pdfListView.pageSliderView.alpha = 0.0;
                self.annotationBar.topToolBar.alpha = 0.0;
                self.annotationBar.drawPencilFuncView.alpha = 0.0;

            }];
        }
    } else {
        CGFloat tPosY = 0;
        if (self.navigationController.navigationBarHidden) {
            [self.navigationController setNavigationBarHidden:NO animated:YES];
            [UIView animateWithDuration:0.3 animations:^{
                self.pdfListView.pageSliderView.alpha = 1.0;
            }];
            CGRect rectStatus = [[UIApplication sharedApplication] statusBarFrame];
            tPosY = self.navigationController.navigationBar.frame.size.height + rectStatus.size.height;

        } else {
            [self.navigationController setNavigationBarHidden:YES animated:YES];
            [UIView animateWithDuration:0.3 animations:^{
                self.pdfListView.pageSliderView.alpha = 0.0;
            }];
        }
    }
    
    if (@available(iOS 11.0, *)) {
        self.pdfListView.frame = CGRectMake(self.view.safeAreaInsets.left, tPosY, self.view.frame.size.width - self.view.safeAreaInsets.left - self.view.safeAreaInsets.right, self.view.frame.size.height - tBottomY - tPosY);
    } else {
        self.pdfListView.frame = CGRectMake(0, tPosY, self.view.frame.size.width, self.view.frame.size.height  - tBottomY - tPosY);
    }

}

- (void)PDFListViewEditNote:(CPDFListView *)pdfListView forAnnotation:(CPDFAnnotation *)annotation {
    if([annotation isKindOfClass:[CPDFLinkAnnotation class]]) {
        [self.annotationBar buttonItemClicked_open:annotation];
    } else {
        CGRect rect = [self.pdfListView convertRect:annotation.bounds fromPage:annotation.page];
        CPDFNoteOpenViewController *noteVC = [[CPDFNoteOpenViewController alloc]initWithAnnotation:annotation];
        noteVC.delegate = self;
        [noteVC showViewController:self inRect:rect];
    }
}

- (void)PDFListViewChangedAnnotationType:(CPDFListView *)pdfListView forAnnotationMode:(CPDFViewAnnotationMode)annotationMode {
    [self.annotationBar reloadData];
}

- (void)PDFListViewPerformAddStamp:(CPDFListView *)pdfView atPoint:(CGPoint)point forPage:(CPDFPage *)page {
    [self.annotationBar addStampAnnotationWithPage:page point:point];
}

- (void)PDFListViewPerformAddImage:(CPDFListView *)pdfView atPoint:(CGPoint)point forPage:(CPDFPage *)page {
    [self.annotationBar addImageAnnotationWithPage:page point:point];
}

- (BOOL)PDFListViewerTouchEndedIsAudioRecordMedia:(CPDFListView *)pdfListView {
    if (CPDFMediaStateAudioRecord == [CPDFMediaManager shareManager].mediaState) {
        [self PDFListViewPerformTouchEnded:self.pdfListView];
        return YES;
    }
    return NO;
}

- (void)PDFListViewPerformCancelMedia:(CPDFListView *)pdfView atPoint:(CGPoint)point forPage:(CPDFPage *)page {
    [CPDFMediaManager shareManager].mediaState = CPDFMediaStateStop;
    self.pdfListView.annotationMode = CPDFViewAnnotationModeNone;
}

- (void)PDFListViewPerformRecordMedia:(CPDFListView *)pdfView atPoint:(CGPoint)point forPage:(CPDFPage *)page {
    NSInteger pageindex = [self.pdfListView.document indexForPage:page];
    [CPDFMediaManager shareManager].mediaState = CPDFMediaStateAudioRecord;
    [CPDFMediaManager shareManager].pageNum = pageindex;
    [CPDFMediaManager shareManager].ptInPdf = point;
    
    if([self.soundPlayBar superview]) {
        if(self.soundPlayBar.soundState == CPDFSoundStatePlay) {
            [self.soundPlayBar stopAudioPlay];
            [self.soundPlayBar removeFromSuperview];
        } else if (self.soundPlayBar.soundState == CPDFSoundStateRecord) {
            [self.soundPlayBar stopRecord];
            [self.soundPlayBar removeFromSuperview];
        }
    }
    
    _soundPlayBar = [[CPDFSoundPlayBar alloc] initWithStyle:self.annotationManage.annotStyle];
    _soundPlayBar.delegate = self;
    [_soundPlayBar showInView:self.pdfListView soundState:CPDFSoundStateRecord];
    [_soundPlayBar startAudioRecord];
}

- (void)PDFListViewPerformPlay:(CPDFListView *)pdfView forAnnotation:(CPDFSoundAnnotation *)annotation {
    NSString *filePath = [annotation mediaPath];
    if (filePath) {
        NSURL *URL = [NSURL fileURLWithPath:filePath];
        
        _soundPlayBar = [[CPDFSoundPlayBar alloc] initWithStyle:self.annotationManage.annotStyle];
        _soundPlayBar.delegate = self;
        [_soundPlayBar showInView:self.pdfListView soundState:CPDFSoundStatePlay];
        [_soundPlayBar setURL:URL];
        [_soundPlayBar startAudioPlay];
        [CPDFMediaManager shareManager].mediaState = CPDFMediaStateVedioPlaying;
    }
}

- (void)PDFListViewPerformSignatureWidget:(CPDFListView *)pdfView forAnnotation:(CPDFSignatureWidgetAnnotation *)annotation {
    [self.annotationBar openSignatureAnnotation:annotation];
}

- (void)PDFListViewEditProperties:(CPDFListView *)pdfListView forAnnotation:(CPDFAnnotation *)annotation {
    [self.annotationBar buttonItemClicked_open:annotation];
}

- (void)PDFListViewCustomMenuClick:(CPDFListView *)pdfView identifier:(NSString *)menuIdentifier {
    if([menuIdentifier isEqualToString:@"PropertyItem"]){
        self.editMode = CPDFEditModeText;
        [self showMenuList];
        [self.toolBar updateButtonState];
    }
}

#pragma mark - CPDFAnnotationBarDelegate

- (void)annotationBarClick:(CPDFAnnotationBar *)annotationBar clickAnnotationMode:(CPDFViewAnnotationMode)annotationMode forSelected:(BOOL)isSelected forButton:(UIButton *)button {
    if(CPDFViewAnnotationModeInk == annotationMode || CPDFViewAnnotationModePencilDrawing == annotationMode) {
        CGFloat tPosY = 0;
        if(isSelected) {
            [self.navigationController setNavigationBarHidden:YES animated:YES];
            [UIView animateWithDuration:0.3 animations:^{
                CGRect frame = self.annotationBar.frame;
                frame.origin.y = self.view.bounds.size.height;
                self.annotationBar.frame = frame;
                self.pdfListView.pageSliderView.alpha = 0.0;
               
                if (@available(iOS 11.0, *)) {
                    self.pdfListView.frame = CGRectMake(self.view.safeAreaInsets.left, tPosY, self.view.frame.size.width - self.view.safeAreaInsets.left - self.view.safeAreaInsets.right, self.view.frame.size.height - self.view.safeAreaInsets.bottom - tPosY);
                } else {
                    self.pdfListView.frame = CGRectMake(0, tPosY, self.view.frame.size.width, self.view.frame.size.height - tPosY);
                }
            }];
        } else {
            [self.navigationController setNavigationBarHidden:NO animated:YES];
            [UIView animateWithDuration:0.3 animations:^{
                CGRect frame = self.annotationBar.frame;
                frame.origin.y = self.view.bounds.size.height-frame.size.height;
                self.annotationBar.frame = frame;
                self.pdfListView.pageSliderView.alpha = 1.0;
            }];
            CGRect rectStatus = [[UIApplication sharedApplication] statusBarFrame];
            tPosY = self.navigationController.navigationBar.frame.size.height + rectStatus.size.height;

            if (@available(iOS 11.0, *)) {
                self.pdfListView.frame = CGRectMake(self.view.safeAreaInsets.left, tPosY, self.view.frame.size.width - self.view.safeAreaInsets.left - self.view.safeAreaInsets.right, self.view.frame.size.height - tPosY- self.annotationBar.frame.size.height);
            } else {
                self.pdfListView.frame = CGRectMake(0, tPosY, self.view.frame.size.width, self.view.frame.size.height - tPosY- tPosY- self.annotationBar.frame.size.height);
            }
        }
    } else if (CPDFViewAnnotationModeSound == annotationMode && !isSelected) {
        if(CPDFSoundStateRecord == self.soundPlayBar.soundState) {
            [self.soundPlayBar stopRecord];

        } else if (CPDFSoundStatePlay== self.soundPlayBar.soundState) {
            [self.soundPlayBar stopAudioPlay];
        }
    }

}

#pragma mark - CPDFNoteOpenViewControllerDelegate

- (void)getNoteOpenViewController:(CPDFNoteOpenViewController *)noteOpenVC content:(NSString *)content isDelete:(BOOL)isDelete {
    if (isDelete) {
        [noteOpenVC.annotation.page removeAnnotation:noteOpenVC.annotation];
        [self.pdfListView setNeedsDisplayForPage:noteOpenVC.annotation.page];
        if([self.pdfListView.activeAnnotations containsObject:noteOpenVC.annotation]) {
            NSMutableArray *activeAnnotations = [NSMutableArray arrayWithArray:self.pdfListView.activeAnnotations];
            [activeAnnotations removeObject:noteOpenVC.annotation];
            [self.pdfListView updateActiveAnnotations:activeAnnotations];
        }
    } else {
        if([noteOpenVC.annotation isKindOfClass:[CPDFMarkupAnnotation class]]) {
            CPDFMarkupAnnotation *markupAnnotation = (CPDFMarkupAnnotation *)noteOpenVC.annotation;
            [markupAnnotation setMarkupText:content?:@""];
        } else {
            noteOpenVC.annotation.contents = content?:@"";
        }
    }
}

#pragma mark - CPDFSoundPlayBarDelegate

- (void)soundPlayBarRecordFinished:(CPDFSoundPlayBar *)soundPlayBar withFile:(NSString *)filePath {
    self.pdfListView.annotationMode = CPDFViewAnnotationModeNone;

    CPDFPage *page = [self.pdfListView.document pageAtIndex:[CPDFMediaManager shareManager].pageNum];
    CPDFSoundAnnotation *annotation = [[CPDFSoundAnnotation alloc] initWithDocument:self.pdfListView.document];
    
    if ([annotation setMediaPath:filePath]) {
        CGRect bounds = annotation.bounds;
        bounds.origin.x = [CPDFMediaManager shareManager].ptInPdf.x-bounds.size.width/2.0;
        bounds.origin.y = [CPDFMediaManager shareManager].ptInPdf.y-bounds.size.height/2.0;
        annotation.bounds = bounds;
        [self.pdfListView addAnnotation:annotation forPage:page];
    }

    [CPDFMediaManager shareManager].mediaState = CPDFMediaStateStop;
    [self.pdfListView stopRecord];
}

- (void)soundPlayBarRecordCancel:(CPDFSoundPlayBar *)soundPlayBar {
    if(CPDFSoundStateRecord == self.soundPlayBar.soundState) {
        [self.pdfListView stopRecord];
    }
    [CPDFMediaManager shareManager].mediaState = CPDFMediaStateStop;
    self.pdfListView.annotationMode = CPDFViewAnnotationModeNone;
}

- (void)soundPlayBarPlayClose:(CPDFSoundPlayBar *)soundPlayBar {
    [CPDFMediaManager shareManager].mediaState = CPDFMediaStateStop;
}

#pragma mark - CPDFToolsViewControllerDelegate

- (void)CPDFToolsViewControllerDismiss:(CPDFToolsViewController *) viewController selectItemAtIndex:(CToolModel)selectIndex {
    if(CToolModelViewer == selectIndex) {
        //viewwer
        [self enterViewerMode];
    }else if(CToolModelEdit == selectIndex) {
        [self enterEditMode];
    }else if(CToolModelAnnotation == selectIndex){
        //Annotation
        [self enterAnnotationMode];
    }
}

#pragma mark - CPDFBOTAViewControllerDelegate

- (void)botaViewControllerDismiss:(CPDFBOTAViewController *)botaViewController {
    [self.navigationController dismissViewControllerAnimated:YES completion:nil];
}

@end
